<?php

use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Migrations\Migration;

class CreateAclTables extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('acl_roles', function (Blueprint $table) {
            $table->increments('id');
            $table->string('name', 50);
            $table->string('description', 255);
            $table->unique('name', 'acl_roles_unique');
        });

        Schema::table('users', function (Blueprint $table)
        {
            $table->foreign('acl_role_id', 'fk_users_roles')
                ->references('id')
                ->on('acl_roles')
                ->onUpdate('cascade')
                ->onDelete('set null');
        });

        Schema::create('acl_permissions', function (Blueprint $table) {
            $table->increments('id');
            $table->string('name', 50);
            $table->string('description', 255);
            $table->unique('name', 'name_unique');
        });

        Schema::create('acl_permissions_acl_roles', function (Blueprint $table) {
            $table->integer('acl_role_id')->unsigned();
            $table->integer('acl_permission_id')->unsigned();

            $table->primary(['acl_role_id', 'acl_permission_id']);

            $table->foreign('acl_role_id', 'fk_acl_pivot_roles')
                ->references('id')
                ->on('acl_roles')
                ->onUpdate('cascade')
                ->onDelete('cascade');
            $table->foreign('acl_permission_id', 'fk_acl_pivot_permissions')
                ->references('id')
                ->on('acl_permissions')
                ->onUpdate('cascade')
                ->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('acl_permissions_acl_roles');

        Schema::dropIfExists('acl_permissions');

        Schema::table('users', function (Blueprint $table) {
            $table->dropForeign('fk_users_roles');
        });

        Schema::dropIfExists('acl_roles');
    }
}
