<?php

namespace AltSolution\Admin\Form\Component;

use AltSolution\Admin\Form\AbstractComponent;

class Header extends AbstractComponent
{
    protected $view = 'header';

    protected function customViewData()
    {
        $title = $this->getOption('title');
        $description = $this->getOption('description');
        $muted = $this->getOption('muted', true);
        $margin = $this->normalizeMargin($this->getOption('margin', [0, 0, 3]));

        return compact('title', 'description', 'muted', 'margin');
    }

    protected function normalizeMargin($margin)
    {
        $result = [
            'top' => 0,
            'right' => 0,
            'bottom' => 0,
            'left' => 0,
        ];

        $value = function($value) {
            // .mt-X, .mr-X, .mb-X, .ml-X
            static $available = [0, 1, 2, 3, 4, 5];

            $margin = (int)$value;
            if (!in_array($margin, $available, true)) {
                throw new \Exception('Invalid margin value: ' . $value);
            }

            return $margin;
        };

        if ($margin) {
            if (is_array($margin)) {
                $cnt = count($margin);
                if ($cnt === 2) {
                    $result['top'] = $result['bottom'] = $value($margin[0]);
                    $result['right'] = $result['left'] = $value($margin[1]);
                } elseif ($cnt === 3) {
                    $result['top'] = $value($margin[0]);
                    $result['right'] = $result['left'] = $value($margin[1]);
                    $result['bottom'] = $value($margin[2]);
                } elseif ($cnt === 4) {
                    $result['top'] = $value($margin[0]);
                    $result['right'] = $value($margin[1]);
                    $result['bottom'] = $value($margin[2]);
                    $result['left'] = $value($margin[3]);
                } else {
                    throw new \Exception('Invalid margin count: ' . $cnt);
                }
            } else {
                $result['top'] = $result['right'] = $result['bottom'] = $result['left'] = $value($margin);
            }
        }

        return $result;
    }
}
