<?php

namespace AltSolution\Admin\Forms;

use AltSolution\Admin\Form;
use AltSolution\Admin\Modules\Menu\MenuItemFormInterface;

class MenuItemForm extends Form\AbstractFactory implements MenuItemFormInterface
{
    public function buildForm(Form\BuilderInterface $builder)
    {
        $builder->form([
            'method' => 'post',
            'action' => route('admin::menu_item_save'),
        ]);

        $builder->hidden('id');
        $builder->hidden('menu_id', [
            'required' => true,
            'rules' => 'integer|exists:menus,id',
        ]);

        $currentLocale = cms_default_locale();
        foreach (cms_locales() as $locale) {
            $builder->text('name_' . $locale, [
                'label' => trans('admin::menu.name_item'),
                'required' => $locale == $currentLocale,
                'rules' => 'string|max:255',
            ]);
        }
        $builder->select('parent_id', [
            'label' => trans('admin::menu.name_parent_id'),
            'choices' => function () use ($builder) {
                yield 0 => trans('admin::menu.root');
                $parent_items = $this->provided('parent_items');
                foreach ($parent_items as $item) {
                    yield $item->id => $item->trans('name');
                }
            },
            'rules' => 'integer', // |exists:menu_items,id
        ]);
        $builder->text('url', [
            'label' => trans('admin::menu.url_item'),
            'readonly' => true,
            'fillable' => true,
            'rules' => 'string|max:255',
        ]);
    }
}