<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Pagination\FilePaginator;
use AltSolution\Admin\Repository\LogsRepository;

class LogsController extends Controller
{
    /**
     * @var LogsRepository
     */
    private $logsRepo;

    /**
     * LogsController constructor.
     * @param LogsRepository $logsRepo
     */
    public function __construct(LogsRepository $logsRepo)
    {
        $this->logsRepo = $logsRepo;
    }

    public function index()
    {
        $this->authorize('permission', 'logs');

        $logs = $this->logsRepo->getLogs();

        $this->layout
            ->setMenuSection('options')
            ->setTitle(trans('admin::log.title'));
        return view('admin::logs.list', compact('logs'));
    }

    public function view($file)
    {
        $this->authorize('permission', 'logs');

        $log = $this->logsRepo->getLog($file);
        if (!$log) {
            abort(404);
        }

        $perPage = config('admin.logs_lines_per_page');
        $paginator = new FilePaginator($log['content'], $log['lines'], $perPage);

        $this->layout
            ->setMenuSection('options')
            ->setTitle($file)
            ->addBreadcrumb(trans('admin::log.title'), route('admin::logs_list'));
        return view('admin::logs.view', compact('file', 'log', 'paginator'));
    }

    public function download($file)
    {
        $this->authorize('permission', 'logs');

        $log = $this->logsRepo->getLogFile($file);
        if (!$log) {
            abort(404);
        }

        return response()->download($log, $file);
    }
}
