<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Option;
use AltSolution\Admin\Seo\SeoRepositoryInterface;
use Illuminate\Http\Request;
use AltSolution\Admin\Form\BuilderInterface;
use Illuminate\Support\Arr;

class OptionsController extends Controller
{
    public function getIndex()
    {
        $this->authorize('permission', 'options');

        $values = app(Option\RepositoryInterface::class)->getAll();
        $forms = [];
        $option = cms_system()->getOption();
        foreach ($option->getSections() as $section) {
            $forms[$section->getName()] = $section->getFormBuilder()
                ->setPrefix($section->getName() . '_')
                ->setDataSource($values)
                ->build();
        }
        $form = cms_construct_form(function() {
            yield [
                'type' => 'form',
                'method' => 'post',
                'buttons' => ['save'],
            ];
        });

        $this->layout
            ->setMenuSection('options')
            ->setTitle(trans('admin::option.title'));
    	return view('admin::options.option', compact('option', 'form', 'forms'));
    }

    public function postIndex(Request $request)
    {
        $this->authorize('permission', 'options');

        $option = cms_system()->getOption();
        foreach ($option->getSections() as $section) {
            $form = $section->getFormBuilder()
                ->setPrefix($section->getName() . '_')
                ->build();

            $data = [];
            foreach ($form->getFields() as $field) {
                $fieldName = $field->getName();
                $data[$fieldName] = $request[$fieldName];
            }

            // TODO: not cool, make custom validation rule
            $this->getValidationFactory()
                ->extend('email_many', function ($attribute, $value, $parameters, $validator) {
                    if (empty($value)) {
                        return true;
                    }
                    $valueArr = explode(',', $value);
                    $valueArr = array_map('trim', $valueArr);
                    foreach ($valueArr as $valueOne) {
                        if (filter_var($valueOne, FILTER_VALIDATE_EMAIL) === false) {
                            $validator->errors()->add($attribute, trans('validation.email', compact('attribute')));
                            break;
                        }
                    }
                    return true;
                });

            $rules = $form->getValidationRules();
            $rules['admin_email_d'] = ['email_many'];
            $validator = $this->getValidationFactory()
                ->make($data, $rules, [], $form->getValidationAttributes());
            // TODO: use Form@validate
            // TODO: partially saved data if validation fails
            if ($validator->fails()) {
                return redirect()->route('admin::options')
                    ->withErrors($validator)
                    ->withInput();
            }

            app(Option\RepositoryInterface::class)->setMany($data);
        }

        $this->layout->addNotify('success', trans('admin::option.saved'));

        return redirect()->route('admin::options');
    }

    public function getSeo()
    {
        $this->authorize('permission', 'options');

        // TODO: move from controller
        $locales = cms_locales();
        $seo = cms_system()->getSeo();
        $values = app(SeoRepositoryInterface::class)->getPagesProperties();

        $forms = [];
        foreach ($seo->getSections() as $section) {
            $forms[$section->getName()] = [];
            foreach ($locales as $locale) {
                // build
                $formBuilder = app(BuilderInterface::class);
                $formBuilder->setDataSource([
                    'seo' => [
                        $section->getName() => Arr::get($values, $section->getName()),
                    ],
                ]);
                foreach ($section->getFields() as $field) {
                    $formBuilder->addMany([$field->getName() => $field->getOptions()]);
                }
                // compile
                $form = $formBuilder
                    ->setPrefix(sprintf('seo.%s.%s.', $section->getName(), $locale))
                    ->build();
                $forms[$section->getName()][$locale] = $form;
            }
        }
        $form = cms_construct_form(function() {
            yield [
                'type' => 'form',
                'method' => 'post',
                'buttons' => ['save'],
            ];
        });

        $this->layout
            ->setMenuSection('options')
            ->setTitle(trans('admin::option.seo_title'));
        return view('admin::options.seo', compact('seo', 'form', 'forms'));
    }

    public function postSeo(Request $request)
    {
        $this->authorize('permission', 'options');

        $locales = cms_locales();
        $seo = cms_system()->getSeo();

        foreach ($seo->getSections() as $section) {
            $formBuilder = app(BuilderInterface::class);
            foreach ($locales as $locale) {
                // build
                foreach ($section->getFields() as $field) {
                    $formBuilder->addMany([
                        $locale . '.' . $field->getName() => $field->getOptions()
                    ]);
                }
            }
            // compile
            $form = $formBuilder->build();

            $data = [];
            $prefix = sprintf('seo.%s.', $section->getName());
            foreach ($form->getFields() as $field) {
                $fieldName = $field->getName();
                Arr::set($data, $fieldName, $request->input($prefix . $fieldName));
            }

            $validator = $this->getValidationFactory()
                ->make($data, $form->getValidationRules(), [], $form->getValidationAttributes());
            // TODO: use Form@validate
            // TODO: partially saved data if validation fails
            if ($validator->fails()) {
                return redirect()->route('admin::seo')
                    ->withErrors($validator)
                    ->withInput();
            }

            app(SeoRepositoryInterface::class)->setPageProperties($section->getName(), $data);
        }

        $this->layout->addNotify('success', trans('admin::option.seo_saved'));

        return redirect()->route('admin::seo');
    }
}