<?php

namespace AltSolution\Admin\Services;

use Illuminate\Support\Arr;
use Illuminate\Support\Str;

final class DashboardService
{
    public function getGeneralInfo()
    {
        $os = [
            'name' => php_uname(),
        ];

        $php = [
            'version' => PHP_VERSION,
            'tmp' => sys_get_temp_dir(),
        ];

        /*
        $extensions = get_loaded_extensions();
        $extensions = array_map('strtolower', $extensions);
        $required = [
            // CORE
            "json",

            // FRAMEWORK
            "openssl",
            "pdo",
            "pdo_mysql",
            "mbstring",
            "tokenizer",
            "xml",
            // "xmlreader"
            // "xmlwriter"
            // "dom"
            // "simplexml"

            // "intl" for mail?

            // CMS
            "gd",
            // "curl"
        ];
        // phpversion('gd')
        // check "xdebug" is not installed
        */

        $laravel = [
            'version' => \Illuminate\Foundation\Application::VERSION,
        ];
        $cms = [
            'version' => \AltSolution\Admin\System\System::VERSION,
        ];


        // assume default connection is mysql
        $conn = \DB::connection(config('database.default'));
        $res = $conn->selectOne($conn->raw('SELECT VERSION() AS v'));

        $mysql = [
            'version' => $res ? $res->v : 'n/a',
        ];

        return compact('os', 'php', 'laravel', 'cms', 'mysql');
    }

    public function getAssetsInfo()
    {
        $path = public_path('vendor/admin');
        $fi = new \SplFileInfo($path);

        $status = 'error';
        $type = 'none';

        if ($fi->isLink()) {
            $status = 'ok';
            $type = 'link';
        } elseif ($fi->isDir()) {
            $status = 'ok';
            $type = 'dir';
        }

        $admin = compact('status', 'type');

        $uploadPath = public_path(config('admin.upload_directory'));
        $uploadFileInfo = new \SplFileInfo($uploadPath);
        $upload = [
            'writable' => $uploadFileInfo->isWritable(),
        ];

        return compact('admin', 'upload');
    }

    public function getProductionInfo()
    {
        $env = config('app.env');
        $live = $env == 'production';
        $debug = config('app.debug');
        $config = app()->configurationIsCached();
        $routes = app()->routesAreCached();

        return compact('env', 'live', 'debug', 'config', 'routes');
    }

    public function getMailInfo()
    {
        $mail = config('mail');

        $mailers = [
            'smtp' => 'SMTP',
            'sendmail' => 'Sendmail',
            'mailgun' => 'Mailgun',
            'mandrill' => 'Mandrill',
            'ses' => 'Amazon SES',
            'sparkpost' => 'SparkPost',
            'postmark' => 'Postmark',
            'log' => 'Log',
            'array' => 'Array',
        ];
        $driver = Arr::get($mailers, $mail['default'], $mail['default']);

        $smtp = null;
        $sendmail = null;
        if ($mail['default'] == 'smtp') {
            $smtp = [
                'host' => Arr::get($mail, 'mailers.smtp.host'),
                'port' => Arr::get($mail, 'mailers.smtp.port'),
                'user' => Arr::get($mail, 'mailers.smtp.username'),
                'enc' => Arr::get($mail, 'mailers.smtp.encryption') ?: 'none',
            ];
        } elseif ($mail['default'] == 'sendmail') {
            $sendmail = [
                'path' => Arr::get($mail, 'mailers.sendmail.path'),
            ];
        }

        if ($mail['from']['name']) {
            $from = $mail['from']['name'] . ' <' . $mail['from']['address'] . '>';
        } else {
            $from = $mail['from']['address'];
        }

        return compact('driver', 'smtp', 'sendmail', 'from');
    }

    public function getSeoInfo()
    {
        $robotsFile = new \SplFileInfo(public_path('robots.txt'));
        $robotsExists = $robotsFile->isFile();

        $robots = [
            'url' => url('robots.txt'),
            'exists' => $robotsExists,
            'allow' => $this->robotsAllowed(),
        ];

        $sitemapFile = new \SplFileInfo(public_path('sitemap.xml'));
        $sitemapExists = $sitemapFile->isFile();
        if ($sitemapExists) {
            $sitemapUpdated = $sitemapFile->getMTime();
        } else {
            $sitemapUpdated = null;
        }
        $sitemap = [
            'url' => url('sitemap.xml'),
            'exists' => $sitemapExists,
            'date' => $sitemapUpdated,
        ];

        return compact('robots', 'sitemap');
    }

    private function robotsAllowed($path = '/', $userAgent = null)
    {
        $robotsTxt = @file(public_path('robots.txt'));
        if (empty($robotsTxt)) {
            // if there isn't a robots, then we're allowed in
            return true;
        }

        $group = null;
        $groups = [];
        foreach ($robotsTxt as $line) {
            $line = trim($line);
            if ($line === '' || preg_match('~^#~u', $line)) {
                // skip blank/commented lines
                continue;
            }
            if (preg_match('~^User-agent:([^#]*)~ui', $line, $match)) {
                if ($group === null || isset($group['rules'])) {
                    // starting new group
                    if ($group !== null) {
                        $groups[] = $group;
                    }
                    $group = [
                        'user-agents' => [],
                        'rules' => [],
                    ];
                }
                $group['user-agents'][] = trim($match[1]);
            }
            if ($group !== null) {
                if (preg_match('~^Allow:([^#]*)~ui', $line, $match)) {
                    $group['rules'][] = ['allow', trim($match[1])];
                }
                if (preg_match('~^Disallow:([^#]*)~ui', $line, $match)) {
                    $group['rules'][] = ['disallow', trim($match[1])];
                }
            }
        }
        if ($group !== null) {
            $groups[] = $group;
        }

        $userAgentRegex = preg_quote('*', '~');
        if ($userAgent) {
            $userAgentRegex .= '|' . preg_quote($userAgent, '~');
        }

        $allow = true;
        foreach ($groups as $group) {
            foreach ($group['user-agents'] as $UA) {

                if (preg_match("~^($userAgentRegex)$~ui", $UA)) {
                    foreach ($group['rules'] as $rule) {

                        list($ruleAccess, $rulePath) = $rule;
                        if (empty($rulePath)) {
                            // ignore empty Allow/Disallow
                            continue;
                        }
                        $ruleRegex = preg_quote($rulePath, '~');

                        if (preg_match("~^$ruleRegex~", $path)) {
                            if ($ruleAccess === 'allow') {
                                $allow = true;
                            } elseif ($ruleAccess === 'disallow') {
                                $allow = false;
                            }
                        }
                    }
                    break 2;
                }
            }
        }

        return $allow;
    }

    /*

    storage dir is writable

    uploads dir is writable

    config('app.key') !== null

    if (Arr::get($config, 'default_connection', true)) {
        DB::connection()->getPdo();
    }
    foreach (Arr::get($config, 'connections', []) as $connection) {
        DB::connection($connection)->getPdo();
    }

    use Illuminate\Filesystem\Filesystem;
    filesystem->isWritable($path);

    filesystem->exists(base_path('.env'));

    try {
        Artisan::call('migrate', ['--pretend' => 'true', '--force' => 'true']);
        $output = Artisan::output();
        return strstr($output, 'Nothing to migrate.');
    } catch (\PDOException $e) {
        $this->databaseError = $e->getMessage();
    }

    filesystem->isDirectory(public_path('storage'));
     */
}
