<?php

namespace AltSolution\Admin\Services;

use AltSolution\Admin\EmailTemplate\TemplateRepository;
use AltSolution\Admin\Models\EmailTemplate as TemplateModel;

final class EmailTemplateService
{
    /**
     * @var TemplateRepository
     */
    private $repository;

    /**
     * @param TemplateRepository $repository
     */
    public function __construct(TemplateRepository $repository)
    {
        $this->repository = $repository;
    }

    /**
     * Add missed email templates and delete non exists
     *
     * @return void
     */
    public function updateEmailTemplates()
    {
        $exists = [];

        $templates = $this->repository->findAll();
        foreach ($templates as $template) {
            $model = TemplateModel::query()
                ->where('name', $template->getName())
                ->first();

            if ($model === null) {
                $model = new TemplateModel();
                $model->name = $template->getName();
                $model->description = $template->getDescription();
                $model->from = '[%NO.REPLY%]';
                $model->to = '[%ADMIN.EMAIL%]';
                $model->subject = '';
                $model->body = '';
                foreach (cms_locales() as $locale) {
                    $model['subject_' . $locale] = '';
                    $model['body_' . $locale] = '';
                }
                $model->layout = false;
                $model->html = false;
                $model->to_admin = false;
                $model->to_d_admin = false;

                $model->save();
            }

            $exists[] = $model->id;
        }

        TemplateModel::query()
            ->whereNotIn('id', $exists)
            ->delete();
    }

}
